# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-11-25 13:38
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('message_sender', '0003_auto_20161124_1357'),
    ]

    operations = [
        migrations.AddField(
            model_name='outbound',
            name='last_sent_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
