# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-02-15 07:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('message_sender', '0006_add_created_at_index'),
    ]

    operations = [
        migrations.CreateModel(
            name='OutboundSendFailure',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('task_id', models.UUIDField()),
                ('initiated_at', models.DateTimeField()),
                ('reason', models.TextField()),
                ('outbound', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='message_sender.Outbound')),
            ],
        ),
    ]
