{% extends "admin/base.html" %}

{% load seed %}

{% block title %}{{ title }} | {{ site_title|default:_('Django site admin') }}{% endblock %}

{% block branding %}
<h1 id="site-name"><a href="{% url 'admin:index' %}">{{ site_header|default:_('Django administration') }}</a></h1>
{% endblock %}

{% block nav-global %}{% endblock %}

{% block footer %}<span style="margin: 0 auto; display: table; color: #808080; font-size: 10px;">v{% current_version %}</span>{% endblock %}
