# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-11-24 11:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('message_sender', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='outbound',
            name='vumi_message_id',
            field=models.CharField(blank=True, db_index=True, max_length=36, null=True),
        ),
    ]
